@@ -27,7 +27,6 @@
 #define HAVE_BSEARCH 1
 #define HAVE_CRYPT_H 1
 #define HAVE_CURSES_H 1
-#define HAVE_CXXABI_H 1
 /* #undef HAVE_BFD_H */
 /* #undef HAVE_NCURSES_H */
 /* #undef HAVE_NDIR_H */
@@ -46,6 +45,7 @@
 #define HAVE_INTTYPES_H 1
 /* #undef HAVE_KQUEUE */
 #define HAVE_LIMITS_H 1
+#define HAVE_LINK_H 1
 #define HAVE_LINUX_UNISTD_H 1
 #define HAVE_LOCALE_H 1
 #define HAVE_MALLOC_H 1
@@ -94,6 +94,7 @@
 #define HAVE_SYS_TYPES_H 1
 #define HAVE_SYS_UN_H 1
 /* #undef HAVE_SYS_VADVISE_H */
+#define HAVE_SYS_STATVFS_H 1
 #define HAVE_TERM_H 1
 /* #undef HAVE_TERMBITS_H */
 #define HAVE_TERMIOS_H 1
@@ -163,6 +164,7 @@
 #define HAVE_FSYNC 1
 #define HAVE_FTIME 1
 #define HAVE_GETADDRINFO 1
+#define HAVE_GETIFADDRS 1
 #define HAVE_GETCWD 1
 #define HAVE_GETHOSTBYADDR_R 1
 /* #undef HAVE_GETHRTIME */
@@ -214,6 +216,8 @@
 #define HAVE_POLL 1
 /* #undef HAVE_PORT_CREATE */
 #define HAVE_POSIX_FALLOCATE 1
+#define HAVE_LINUX_FALLOC_H 1
+/* #undef HAVE_FALLOCATE */
 #define HAVE_PREAD 1
 #define HAVE_PAUSE_INSTRUCTION 1
 /* #undef HAVE_FAKE_PAUSE_INSTRUCTION */
@@ -395,7 +399,7 @@
 
 #define SOCKET_SIZE_TYPE socklen_t
 
-#define HAVE_MBSTATE_T
+#define HAVE_MBSTATE_T 1
 
 #define MAX_INDEXES 64
 
@@ -430,15 +434,15 @@
 #define HAVE_WCTYPE_H 1
 #define HAVE_WCHAR_H 1
 #define HAVE_LANGINFO_H 1
-#define HAVE_MBRLEN
+#define HAVE_MBRLEN 1
 /* #undef HAVE_MBSCMP */
-#define HAVE_MBSRTOWCS
-#define HAVE_WCRTOMB
-#define HAVE_MBRTOWC
-#define HAVE_WCSCOLL
-#define HAVE_WCSDUP
-#define HAVE_WCWIDTH
-#define HAVE_WCTYPE
+#define HAVE_MBSRTOWCS 1
+#define HAVE_WCRTOMB 1
+#define HAVE_MBRTOWC 1
+#define HAVE_WCSCOLL 1
+#define HAVE_WCSDUP 1
+#define HAVE_WCWIDTH 1
+#define HAVE_WCTYPE 1
 #define HAVE_ISWLOWER 1
 #define HAVE_ISWUPPER 1
 #define HAVE_TOWLOWER 1
@@ -452,7 +456,7 @@
 #define HAVE_STRCASECMP 1
 #define HAVE_STRNCASECMP 1
 #define HAVE_STRDUP 1
-#define HAVE_LANGINFO_CODESET 
+#define HAVE_LANGINFO_CODESET  1
 #define HAVE_TCGETATTR 1
 #define HAVE_FLOCKFILE 1
 
@@ -469,8 +473,8 @@
 /* #undef HAVE_SOLARIS_STYLE_GETHOST */
 
 /* #undef MY_ATOMIC_MODE_DUMMY */
-/* #undef MY_ATOMIC_MODE_RWLOCKS */
 #define HAVE_GCC_ATOMIC_BUILTINS 1
+/* #undef HAVE_GCC_C11_ATOMICS */
 /* #undef HAVE_SOLARIS_ATOMIC */
 #define HAVE_DECL_SHM_HUGETLB 1
 #define HAVE_LARGE_PAGES 1
@@ -531,7 +535,6 @@
 #define ENABLED_LOCAL_INFILE 1
 #define ENABLED_PROFILING 1
 /* #undef EXTRA_DEBUG */
-/* #undef BACKUP_TEST */
 /* #undef CYBOZU */
 /* #undef USE_SYMDIR */
 
@@ -587,7 +590,8 @@
 #define HAVE_CHARSET_utf32 1
 #define HAVE_UCA_COLLATIONS 1
 #define HAVE_COMPRESS 1
-
+/* #undef HAVE_EncryptAes128Ctr */
+/* #undef HAVE_EncryptAes128Gcm */
 
 /*
   Stuff that always need to be defined (compile breaks without it)
@@ -604,15 +608,11 @@
 #define WITH_MYISAM_STORAGE_ENGINE 1
 #define WITH_MYISAMMRG_STORAGE_ENGINE 1
 #define WITH_HEAP_STORAGE_ENGINE 1
-#define WITH_INNOBASE_STORAGE_ENGINE 1
+/* #undef WITH_INNOBASE_STORAGE_ENGINE */
 #define WITH_XTRADB_STORAGE_ENGINE 1
 #define WITH_CSV_STORAGE_ENGINE 1
 #define WITH_PARTITION_STORAGE_ENGINE 1
 #define WITH_PERFSCHEMA_STORAGE_ENGINE 1
-/* #undef WITH_NDBCLUSTER_STORAGE_ENGINE */
-#if (WITH_NDBCLUSTER_STORAGE_ENGINE) && !defined(EMBEDDED_LIBRARY)
-# define HAVE_NDB_BINLOG 1
-#endif
 #define WITH_ARIA_STORAGE_ENGINE 1
 #define USE_ARIA_FOR_TMP_TABLES 1
 
@@ -628,26 +628,32 @@
 /* #undef SO_EXT */
 
 #define MYSQL_VERSION_MAJOR 10
-#define MYSQL_VERSION_MINOR 0
-#define MYSQL_VERSION_PATCH 17
+#define MYSQL_VERSION_MINOR 1
+#define MYSQL_VERSION_PATCH 6
 #define MYSQL_VERSION_EXTRA ""
 
 #define PACKAGE "mysql"
 #define PACKAGE_BUGREPORT ""
 #define PACKAGE_NAME "MySQL Server"
-#define PACKAGE_STRING "MySQL Server 10.0.17"
+#define PACKAGE_STRING "MySQL Server 10.1.6"
 #define PACKAGE_TARNAME "mysql"
-#define PACKAGE_VERSION "10.0.17"
-#define VERSION "10.0.17"
+#define PACKAGE_VERSION "10.1.6"
+#define VERSION "10.1.6"
 #define PROTOCOL_VERSION 10
 
-#define MALLOC_LIBRARY "bundled jemalloc"
+#define MALLOC_LIBRARY "system jemalloc"
 
 /* time_t related defines */
 
 #define SIZEOF_TIME_T 8
 /* #undef TIME_T_UNSIGNED */
 
+#ifndef EMBEDDED_LIBRARY
+/* #undef WSREP_INTERFACE_VERSION */
+#define WITH_WSREP 1
+#define WSREP_PROC_INFO 1
+#endif
+
 #ifdef _AIX
 /*
   AIX includes inttypes.h from sys/types.h
