@@ -11,18 +11,22 @@
 #include <custom_conf.h>
 #else
 #define PROTOCOL_VERSION		10
-#define MYSQL_SERVER_VERSION		"10.0.38-MariaDB"
-#define MYSQL_BASE_VERSION		"mysqld-10.0"
-#define MARIADB_BASE_VERSION		"mariadb-10.0"
+#define MYSQL_SERVER_VERSION		"10.1.6-MariaDB"
+#define MYSQL_BASE_VERSION		"mysqld-10.1"
+#define MARIADB_BASE_VERSION		"mariadb-10.1"
 #define MYSQL_SERVER_SUFFIX_DEF		""
 #define FRM_VER				6
-#define MYSQL_VERSION_ID		100038
+#define MYSQL_VERSION_ID		100106
 #define MYSQL_PORT			3306
 #define MYSQL_PORT_DEFAULT		0
 #define MYSQL_UNIX_ADDR			"/tmp/mysql.sock"
 #define MYSQL_CONFIG_NAME		"my"
 #define MYSQL_COMPILATION_COMMENT	"MariaDB Server"
 
+#ifdef WITH_WSREP
+#define WSREP_PATCH_VERSION             "wsrep_25.10"
+#endif
+
 /* mysqld compile time options */
 #endif /* _CUSTOMCONFIG_ */
 
